
#include "OneWireNg_CurrentPlatform.h"
#include "drivers/DSTherm.h"
#include "utils/Placeholder.h"

#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>
Adafruit_MPU6050 mpu;

int ledR = 1;
#define OW_PIN          0

/*
   Set to true for parasitically powered sensors.
*/
#define PARASITE_POWER  false


static Placeholder<OneWireNg_CurrentPlatform> _ow;

/* returns false if not supported */
static bool printId(const OneWireNg::Id& id)
{
  const char *name = DSTherm::getFamilyName(id);

  Serial.print(id[0], HEX);
  for (size_t i = 1; i < sizeof(OneWireNg::Id); i++) {
    Serial.print(':');
    Serial.print(id[i], HEX);
  }
  if (name) {
    Serial.print(" -> ");
    Serial.print(name);
  }
  Serial.println();

  return (name != NULL);
}

static void printScratchpad(const DSTherm::Scratchpad& scrpd)
{
  const uint8_t *scrpd_raw = scrpd.getRaw();


  long temp = scrpd.getTemp();
  Serial.print("Temp DS18B20:");
  if (temp < 0) {
    temp = -temp;
    Serial.print('-');
  }
  Serial.print(temp / 1000);
  Serial.print('.');
  Serial.print(temp % 1000);
  Serial.print(" C");

  Serial.println();
}

void setup()
{
#ifdef PWR_CTRL_PIN
# ifndef CONFIG_PWR_CTRL_ENABLED
#  error "CONFIG_PWR_CTRL_ENABLED needs to be enabled"
# endif
  new (&_ow) OneWireNg_CurrentPlatform(OW_PIN, PWR_CTRL_PIN, false);
#else
  new (&_ow) OneWireNg_CurrentPlatform(OW_PIN, false);
#endif
  DSTherm drv(_ow);

  Serial.begin(115200);

#if (CONFIG_MAX_SRCH_FILTERS > 0)
  static_assert(CONFIG_MAX_SRCH_FILTERS >= DSTherm::SUPPORTED_SLAVES_NUM,
                "CONFIG_MAX_SRCH_FILTERS too small");

  drv.filterSupportedSlaves();
#endif

#ifdef COMMON_RES
 
  drv.writeScratchpadAll(0, 0, COMMON_RES);

  drv.copyScratchpadAll(PARASITE_POWER);
#endif


  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  Serial.println("MPU6050 Found!");




  mpu.setAccelerometerRange(MPU6050_RANGE_8_G);
  Serial.print("Accelerometer range set to: ");
  switch (mpu.getAccelerometerRange()) {
    case MPU6050_RANGE_2_G:
      Serial.println("+-2G");
      break;
    case MPU6050_RANGE_4_G:
      Serial.println("+-4G");
      break;
    case MPU6050_RANGE_8_G:
      Serial.println("+-8G");
      break;
    case MPU6050_RANGE_16_G:
      Serial.println("+-16G");
      break;
  }
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);
  Serial.print("Gyro range set to: ");
  switch (mpu.getGyroRange()) {
    case MPU6050_RANGE_250_DEG:
      Serial.println("+- 250 deg/s");
      break;
    case MPU6050_RANGE_500_DEG:
      Serial.println("+- 500 deg/s");
      break;
    case MPU6050_RANGE_1000_DEG:
      Serial.println("+- 1000 deg/s");
      break;
    case MPU6050_RANGE_2000_DEG:
      Serial.println("+- 2000 deg/s");
      break;
  }

  mpu.setFilterBandwidth(MPU6050_BAND_21_HZ);
  Serial.print("Filter bandwidth set to: ");
  switch (mpu.getFilterBandwidth()) {
    case MPU6050_BAND_260_HZ:
      Serial.println("260 Hz");
      break;
    case MPU6050_BAND_184_HZ:
      Serial.println("184 Hz");
      break;
    case MPU6050_BAND_94_HZ:
      Serial.println("94 Hz");
      break;
    case MPU6050_BAND_44_HZ:
      Serial.println("44 Hz");
      break;
    case MPU6050_BAND_21_HZ:
      Serial.println("21 Hz");
      break;
    case MPU6050_BAND_10_HZ:
      Serial.println("10 Hz");
      break;
    case MPU6050_BAND_5_HZ:
      Serial.println("5 Hz");
      break;
  }


  pinMode(ledR, OUTPUT);

}

void loop()
{
  DSTherm drv(_ow);
  Placeholder<DSTherm::Scratchpad> _scrpd;

  /* convert temperature on all sensors connected... */
  drv.convertTempAll(DSTherm::SCAN_BUS, PARASITE_POWER);

  /* ...and read them one-by-one */
  for (const auto& id : (OneWireNg&)_ow) {
    if (printId(id)) {
      if (drv.readScratchpad(id, &_scrpd) == OneWireNg::EC_SUCCESS)
        printScratchpad(_scrpd);
      else
        Serial.println("  Invalid CRC!");
    }
  }

  Serial.println("----------");


  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);

  /* Print out the values */
  Serial.print("Acceleration X: ");
  Serial.print(a.acceleration.x);
  Serial.print(", Y: ");
  Serial.print(a.acceleration.y);
  Serial.print(", Z: ");
  Serial.print(a.acceleration.z);
  Serial.println(" m/s^2");

  Serial.print("Rotation X: ");
  Serial.print(g.gyro.x);
  Serial.print(", Y: ");
  Serial.print(g.gyro.y);
  Serial.print(", Z: ");
  Serial.print(g.gyro.z);
  Serial.println(" rad/s");

  Serial.print("Temperature: ");
  Serial.print(temp.temperature);
  Serial.println(" degC");

  Serial.println("----------");

  digitalWrite(ledR, HIGH);   // turn the LED Red on
  delay(400);
  digitalWrite(ledR, LOW);   // turn the LED Red off
  delay(400);



}
